import request from '@/utils/request'

// 查询品类信息列表
export function listCategory(query) {
  return request({
    url: '/product/category/list',
    method: 'get',
    params: query
  })
}

// 查询品类信息详细
export function getCategory(id) {
  return request({
    url: '/product/category/' + id,
    method: 'get'
  })
}
// 查询品类共有元素
export function getTemplateAttribute(id) {
  return request({
    url: '/product/category/getTemplateAttribute/' + id,
    method: 'get'
  })
}

// 查询品类信息详细
export function parentCommonElement(id) {
  return request({
    url: '/product/category/parentCommonElement/' + id,
    method: 'get'
  })
}

// 新增品类信息
export function addCategory(data) {
  return request({
    url: '/product/category',
    method: 'post',
    data: data
  })
}

// 修改品类信息
export function updateCategory(data) {
  return request({
    url: '/product/category/'+data.id,
    method: 'put',
    data: data
  })
}

// 删除品类信息
export function delCategory(id) {
  return request({
    url: '/product/category/' + id,
    method: 'delete'
  })
}
